﻿using ConsoleApp2.Entities;
using ConsoleApp2.Options;
using Microsoft.Extensions.Logging;
using MySqlConnector;

namespace ConsoleApp2.Services;

public class DatabaseOutputService
{
    private readonly ILogger _logger;

    public DatabaseOutputService(ILogger<DatabaseOutputService> logger)
    {
        _logger = logger;
    }

    public async Task ExecuteAsync(DataRecordQueue records, DatabaseOptions options, CancellationToken stoppingToken = default)
    {
        var count = records.Count;
        var output = new MySqlDestination(new MySqlConnectionStringBuilder()
        {
            Server = options.Host,
            Port = options.Port,
            Database = options.Database,
            UserID = options.User,
            Password = options.Password,
            ConnectionTimeout = 120,
        }.ConnectionString, _logger); // TODO: 加入DI
        
        while (records.TryDequeue(out var record) && !stoppingToken.IsCancellationRequested)
        {
            await output.WriteRecordAsync(record);
        }
        
        await output.FlushAsync();

        _logger.LogInformation("Flush {Count} records to database.", count);
    }
}